/*    
 * Copyright (c) 2014 Samsung Electronics Co., Ltd.   
 * All rights reserved.   
 *   
 * Redistribution and use in source and binary forms, with or without   
 * modification, are permitted provided that the following conditions are   
 * met:   
 *   
 *     * Redistributions of source code must retain the above copyright   
 *        notice, this list of conditions and the following disclaimer.  
 *     * Redistributions in binary form must reproduce the above  
 *       copyright notice, this list of conditions and the following disclaimer  
 *       in the documentation and/or other materials provided with the  
 *       distribution.  
 *     * Neither the name of Samsung Electronics Co., Ltd. nor the names of its  
 *       contributors may be used to endorse or promote products derived from  
 *       this software without specific prior written permission.  
 *  
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS  
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT  
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR  
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT  
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,  
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT  
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,  
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY  
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT  
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE  
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

var GALL_THM_RQST = 'gallery-thumbnail-req';
var GALL_THM_RESP = 'gallery-thumbnail-rsp';
var GALL_IMG_RQST = 'gallery-image-req';
var GALL_IMG_RESP = 'gallery-image-rsp';
var GALL_PUSH_IND = 'gallery-push-ind';
var GALL_NEXT_IND = 'gallery-next-ind';

var gLatestOffset = -1;

/* Alert message with toast popup of TAU */
function toastAlert(msg) {
    var toastMsg = document.getElementById("popupToastMsg");
    toastMsg.innerHTML = msg;
    // set toast message content with msg and openPopup
    tau.openPopup('#popupToast');
    console.log(msg);
}

/* callback: SAP init successe */
var sapinitsuccesscb = {
    onsuccess : function() {
        console.log('Succeed to connect');
        requestList();
    },
    ondevicestatus : function(status) {
        if (status == "DETACHED") {
            toastAlert('Detached remote peer device');
            clearList(false, true);
        } else if (status == "ATTACHED") {
            reconnect();
        }
    },
    onlistener : function(respData) {
        switch (respData.msgId) {
        case GALL_PUSH_IND:
            gLatestOffset = -1;
            $('.ui-listview').empty();
            requestList();
            return true;
        case GALL_NEXT_IND:
            requestList();
            return true;
        default:
            return false;
        }
    }
};

/* Initialize main page */
function initialize() {
    sapInit(sapinitsuccesscb, function(err) {
        console.log(err.name);
        if (err.name == "PEER_DISCONNECTED") {
            toastAlert(err.message);
            clearList(true, true);
        } else {
            toastAlert('Failed to connect to service');
            clearList(true, true);
        }
    });
}

/* Request and get list of image from host device */
function requestList() {
    var reqData = {
        'msgId' : GALL_THM_RQST,
        'offset' : gLatestOffset
    };
    sapRequest(reqData, function(respData) {
        var count = respData.count;
        var list = respData.list;

        $('.ui-listview li:last-child').remove();

        for ( var i = 0; i < count; i++) {
            var id = list[i].id,
                image = list[i].image,
                name = list[i].name,
                nameStr = (name.length > 15) ? (name.substring(0, 11) + '...') : name;

            if (id <= gLatestOffset)
                continue;

            $('.ui-listview').append(
                    '<li><a onclick="requestImage(' + id + ');"><img src="data:image/jpeg;base64,' + image + '" width="30" height="30"/> ' + nameStr
                            + '</a></li>');
        }
        $('.ui-listview').append('<li><a href="#" onclick="requestList();">More...</a></li>');

        gLatestOffset = list[respData.count - 1].id;
        var snaplistEl = document.getElementsByClassName('ui-snap-listview')[0];
        if (snaplistEl){
        	var snaplistWidget = tau.widget.SnapListview(snaplistEl);
	        snaplistWidget.refresh();
        }
    }, function(err) {
        console.log('Failed to get list.');
    });
}

function reconnect() {
    $('.ui-listview').empty();
    sapFindPeer(function(){
        console.log('Succeed to reconnect');
    }, function(err){
        toastAlert('Failed to reconnect to service');
        clearList(true, true);
    });
}

function clearList(reconnect, snap) {
    console.log('clear List');
    gLatestOffset = -1;
    $('.ui-listview').empty();
    if (reconnect) {
        $('.ui-listview').append('<li><a href="#" onclick="reconnect();">Connect</a></li>');
    } else {
        $('.ui-listview').append('<li>BT Disconnected. Connection waiting...</li>');
    }
    if (snap) {
        var snaplistEl = document.getElementsByClassName('ui-snap-listview')[0];
        if (snaplistEl) {
            var snaplistWidget = tau.widget.SnapListview(snaplistEl);
            snaplistWidget.refresh();
        }
    }
}
    
function requestImage(id) {
    var reqData = {
        'msgId' : GALL_IMG_RQST,
        'id' : id,
        'width' : 320,
        'height' : 320
};

    sapRequest(reqData, function(respData) {
        var imageView = document.getElementById("imageView");
        /* set image tag for imageView and changePage to image page */
        imageView.innerHTML = '<img id="viewImage" src="data:image/jpeg;base64,' + respData.image.image + '"/>';
        tau.changePage('#imagePage');
    }, function(err) {
        toastAlert('Failed to get image');
    });
}

(function() {
    window.addEventListener('tizenhwkey', function(e) {
           /* For the flick down gesture */
        if (e.keyName == "back") {
            var page = document.getElementsByClassName('ui-page-active')[0],
                pageid = page ? page.id : " ";
            if (pageid === "main"){
                 /* When a user flicks down, the application exits */
                tizen.application.getCurrentApplication().exit();
            }
            else {
                window.history.back();
            }
        }
    });
    window.addEventListener('load', function(ev) {
        initialize();
    });
}());

(function(tau) {
    var toastPopup = document.getElementById('popupToast');
    toastPopup.addEventListener('popupshow', function(ev){
        setTimeout(function(){tau.closePopup();}, 3000);
    }, false);
})(window.tau);
