/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd. All rights reserved. 
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that 
 * the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice, 
 *       this list of conditions and the following disclaimer. 
 *     * Redistributions in binary form must reproduce the above copyright notice, 
 *       this list of conditions and the following disclaimer in the documentation and/or 
 *       other materials provided with the distribution. 
 *     * Neither the name of Samsung Electronics Co., Ltd. nor the names of its contributors may be used to endorse or 
 *       promote products derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package com.samsung.android.sdk.accessory.example.galleryconsumer;

import java.util.List;

import android.Manifest;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Base64;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.samsung.android.sdk.accessory.example.galleryconsumer.GalleryConsumerService.ImageListReceiver;
import com.samsung.android.sdk.accessory.*;

public class GalleryConsumerActivity extends Activity implements OnClickListener, ImageListReceiver {
    private static final String TAG = "GalleryConsumerActivity";

    private static final int MSG_THUMBNAIL_RECEIVED = 1986;
    private static final int MSG_IMAGE_RECEIVED = 1987;
    private static final int INITIAL_IMAGE_INDEX = -1;
    private static Object mListLock = new Object();

    private int mNextIndex = INITIAL_IMAGE_INDEX;
    private int mIndexThumb1 = INITIAL_IMAGE_INDEX;
    private int mIndexThumb2 = INITIAL_IMAGE_INDEX;
    private int mIndexThumb3 = INITIAL_IMAGE_INDEX;
    private GalleryConsumerService mBackendService = null;
    private List<ImageStructure> mDTBListReceived = null;
    private ImageStructure mImage = null;
    private String mDownscaledImage = "";
    private Button mFetch;
    private Button mNext;
    private Button mConnect;
    private Button mClose;
    private ImageView mThumbView1;
    private ImageView mThumbView2;
    private ImageView mThumbView3;
    private TextView mTxt1;
    private TextView mTxt2;
    private TextView mTxt3;
    private Handler mHandler = new Handler(new ImageReceivedHandlerCallback()) {
    };

    private SAAgentV2.RequestAgentCallback mAgentCallback = new SAAgentV2.RequestAgentCallback() {
        @Override
        public void onAgentAvailable(SAAgentV2 agent) {
            mBackendService = (GalleryConsumerService)agent;
            if(mBackendService != null) {
                mBackendService.registerImageReciever(GalleryConsumerActivity.this);
            }
        }

        @Override
        public void onError(int errorCode, String message) {
            Log.e(TAG, "Agent initialization error: " + errorCode + ". ErrorMsg: " + message);
        }
    };


    @Override
    protected void onResume() {
        super.onResume();
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        initializeService();
        mFetch = (Button) findViewById(R.id.btnFetch);
        mFetch.setOnClickListener(this);
        mNext = (Button) findViewById(R.id.btnNext);
        mNext.setOnClickListener(this);
        mConnect = (Button) findViewById(R.id.btnConnect);
        mConnect.setOnClickListener(this);
        mClose = (Button) findViewById(R.id.btnClose);
        mClose.setOnClickListener(this);
        mThumbView1 = (ImageView) findViewById(R.id.imageView1);
        mThumbView1.setOnClickListener(this);
        mThumbView2 = (ImageView) findViewById(R.id.imageView2);
        mThumbView2.setOnClickListener(this);
        mThumbView3 = (ImageView) findViewById(R.id.imageView3);
        mThumbView3.setOnClickListener(this);
        mTxt1 = (TextView) findViewById(R.id.textView1);
        mTxt1.setOnClickListener(this);
        mTxt2 = (TextView) findViewById(R.id.textView2);
        mTxt2.setOnClickListener(this);
        mTxt3 = (TextView) findViewById(R.id.textView3);
        mTxt3.setOnClickListener(this);

        // set permission of storage
        if (Build.VERSION.SDK_INT > Build.VERSION_CODES.LOLLIPOP_MR1) {
            if (checkSelfPermission(Manifest.permission.WRITE_EXTERNAL_STORAGE) == PackageManager.PERMISSION_DENIED)
                requestPermissions(new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE}, 1);
        }
    }

    protected void onDestroy() {
        Log.i(TAG, "onDestroy");
        closeConnection();
        destroyService();
        super.onDestroy();
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.btnConnect: {
                if (mBackendService != null) {
                    mBackendService.findPeers();
                }
            }
                break;
            case R.id.btnFetch: {
                if (mBackendService != null) {
                    requestThumbnails();
                }
            }
                break;
            case R.id.btnNext: {
                if (mBackendService != null) {
                    requestNext();
                }
            }
                break;
            case R.id.btnClose: {
                if (mBackendService != null) {
                    closeConnection();
                }
                clearThumbnails();
                mNextIndex = INITIAL_IMAGE_INDEX;
            }
                break;
            case R.id.imageView1: {
                requestImage(mIndexThumb1);
            }
                break;
            case R.id.imageView2: {
                requestImage(mIndexThumb2);
            }
                break;
            case R.id.imageView3: {
                requestImage(mIndexThumb3);
            }
                break;
            default:
        }
    }

    @Override
    public void onImageReceived(ImageStructure img) {
        mImage = img;
        Message msg = Message.obtain();
        msg.what = MSG_IMAGE_RECEIVED;
        mHandler.sendMessage(msg);
    }

    @Override
    public void onThumbnailsReceived(List<ImageStructure> uListReceived) {
        synchronized (mListLock) {
            // consumer can use this list to hold previous thumbnails as cache
            Log.d(TAG, "onThumbnailsReceived Enter");
            mDTBListReceived = uListReceived;
            Message msg = Message.obtain();
            msg.what = MSG_THUMBNAIL_RECEIVED;
            mHandler.sendMessage(msg);
        }
    }

    @Override
    public void onPeerFound(SAPeerAgent uRemoteAgent) {
        if (uRemoteAgent != null) {
            if (mBackendService != null) {
                mBackendService.establishConnection(uRemoteAgent);
            }
        } else {
            Toast.makeText(getApplicationContext(), R.string.NoPeersFound, Toast.LENGTH_LONG).show();
        }
    }

    @Override
    public void onServiceConnectionResponse(int result) {
        if (result == SAAgent.CONNECTION_SUCCESS) {
            Toast.makeText(getApplicationContext(), R.string.ConnectionEstablishedMsg, Toast.LENGTH_LONG).show();
        }
    }

    @Override
    public void onServiceConnectionLost(int errorcode) {
        mHandler.post(new Runnable() {
            @Override
            public void run() {
                Toast.makeText(getBaseContext(), R.string.ConnectionLostMsg, Toast.LENGTH_SHORT).show();
            }
        });
    }

    @Override
    public void onThumbnailsPush() {
        if (mBackendService != null) {
            requestThumbnails();
        }
    }

    @Override
    public void onThumbnailsNext() {
        if (mBackendService != null) {
            requestNext();
        }
    }

    void initializeService() {
        SAAgentV2.requestAgent(getApplicationContext(), GalleryConsumerService.class.getName(), mAgentCallback);
    }

    void destroyService() {
        if (mBackendService != null) {
            mBackendService.releaseAgent();
            mBackendService = null;
        }
    }

    void closeConnection() {
        if (mBackendService != null) {
            mBackendService.closeConnection();
        }
    }

    private void requestThumbnails() {
        mBackendService.requestThumbNail(INITIAL_IMAGE_INDEX);
        mNextIndex = INITIAL_IMAGE_INDEX;
    }

    private void requestNext() {
        if (mNextIndex != -1)
            mBackendService.requestThumbNail(mNextIndex);
        else
            requestThumbnails();
    }

    private void requestImage(int index) {
        if (index != INITIAL_IMAGE_INDEX) {
            mBackendService.requestImage(index);
        }
    }

    private void clearThumbnails() {
        mIndexThumb1 = INITIAL_IMAGE_INDEX;
        mIndexThumb2 = INITIAL_IMAGE_INDEX;
        mIndexThumb3 = INITIAL_IMAGE_INDEX;
        mThumbView1.setImageResource(R.drawable.ic_launcher);
        mThumbView2.setImageResource(R.drawable.ic_launcher);
        mThumbView3.setImageResource(R.drawable.ic_launcher);
        mTxt1.setText(" ");
        mTxt2.setText(" ");
        mTxt3.setText(" ");
    }

    private boolean handleRecievedImage() {
        // parse the structure and render on Image VIEW
        mDownscaledImage = mImage.mData;
        byte[] decodedstream;

        try
        {
            decodedstream = Base64.decode(mDownscaledImage, Base64.NO_WRAP);
        } catch(IllegalArgumentException e) {
            Log.e(TAG, "handleReceivedThumbnails: mDownscaledImage "+mDownscaledImage);
            Log.e(TAG, "handleReceivedThumbnails decode failed "+e.getMessage());
            return false;
        }

        Bitmap bitmap = BitmapFactory.decodeByteArray(decodedstream, 0, decodedstream.length);
        // try new full screen approach
        final AlertDialog.Builder thumbDialog = new AlertDialog.Builder(GalleryConsumerActivity.this);
        ImageView thumbView = new ImageView(GalleryConsumerActivity.this);
        thumbView.setMinimumWidth(320);
        thumbView.setMinimumWidth(240);
        thumbView.setBackgroundResource(00000000);
        if (bitmap != null) {
            thumbView.setImageBitmap(bitmap);
        }
        LinearLayout layout = new LinearLayout(GalleryConsumerActivity.this);
        layout.setOrientation(LinearLayout.VERTICAL);
        layout.addView(thumbView);
        thumbDialog.setView(layout);
        thumbDialog.show();
        Toast.makeText(getApplicationContext(), mImage.mDisplayname, Toast.LENGTH_LONG).show();
        return true;
    }

    private boolean handleReceivedThumbnails() {
        synchronized (mListLock) {
            int size = mDTBListReceived.size();
            if (!mDTBListReceived.isEmpty()) {
                // last elements id for reference
                try {
                    mNextIndex = Integer.parseInt(mDTBListReceived.get(size - 1).mId);
                } catch (IndexOutOfBoundsException e) {
                    e.printStackTrace();
                }
                while (size > 0) {
                    ImageStructure thb1 = mDTBListReceived.get(size - 1);
                    byte[] decodedstream;
                    try
                    {
                        decodedstream = Base64.decode(thb1.mData, Base64.NO_WRAP);
                    } catch(IllegalArgumentException e) {
                        Log.e(TAG, "handleReceivedThumbnails: mData "+thb1.mData);
                        Log.e(TAG, "handleReceivedThumbnails decode failed "+e.getMessage());
                        return false;
                    }
                    Bitmap bitmap = BitmapFactory.decodeByteArray(
                                decodedstream, 0, decodedstream.length);
                    if (bitmap != null) {
                        switch (size) {
                            case 3: {
                                mThumbView1.setImageBitmap(bitmap);
                                mIndexThumb1 = Integer.parseInt(thb1.mId);
                                mTxt1.setText(thb1.mDisplayname);
                            }
                                break;
                            case 2: {
                                mThumbView2.setImageBitmap(bitmap);
                                mIndexThumb2 = Integer.parseInt(thb1.mId);
                                mTxt2.setText(thb1.mDisplayname);
                            }
                                break;
                            case 1: {
                                mThumbView3.setImageBitmap(bitmap);
                                mIndexThumb3 = Integer.parseInt(thb1.mId);
                                mTxt3.setText(thb1.mDisplayname);
                            }
                                break;
                        }
                    }
                    size--;
                }
            }
            return true;
        }
    }

    class ImageReceivedHandlerCallback implements Handler.Callback {
        @Override
        public boolean handleMessage(Message msg) {
            if (msg.what == MSG_THUMBNAIL_RECEIVED) {
                clearThumbnails();
                handleReceivedThumbnails();
            } else if (msg.what == MSG_IMAGE_RECEIVED) {
                handleRecievedImage();
            }
            return true;
        }
    }
}
