/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

 #include "galleryprovider.h"

#include <media_content.h>

#include <stdio.h>
#include <sys/types.h>
#include <dirent.h>

#include <glib.h>

#include "composer.h"
#include <string.h>

typedef struct appdata {
	Evas_Object *win;
	Evas_Object *conform;
	Evas_Object *label;
	Evas_Object *naviframe;
	Eext_Circle_Surface *circle_surface;
	Evas_Object *circle_genlist;
} appdata_s;

GList *images = NULL;
GList *tb_images = NULL;

long total_img_size;
static int counter;

static void win_delete_request_cb(void *data, Evas_Object *obj, void *event_info)
{
	ui_app_exit();
}

void handle_new_request(void)
{
	counter = 0;
}

long read_img_size(char *file_path)
{
	FILE *fp;
	long file_size;
	fp = fopen(file_path, "r");

	dlog_print(DLOG_DEBUG, TAG, "new image file path:%s", file_path);

	if (fp == NULL) {
		dlog_print(DLOG_ERROR, TAG, "could not read image");
		return 0;
	}

	fseek(fp, 0L, SEEK_END);
	file_size = ftell(fp);

	fclose(fp);

	return file_size;
}

char *read_image(char *file_path, size_t *size)
{
	FILE *fp;
	char *out_data;
	int file_size;
	int result;

	fp = fopen(file_path, "r");

	dlog_print(DLOG_DEBUG, TAG, "new imaged file path:%s", file_path);

	if (fp == NULL) {
		dlog_print(DLOG_ERROR, TAG, "could not read image");
		*size = 0;
		return NULL;
	}

	fseek(fp, 0L, SEEK_END);
	file_size = ftell(fp);

	if (file_size == 0) {
		*size = 0;
		return NULL;
	}

	out_data = (char *)malloc(file_size);

	fseek(fp, 0L, SEEK_SET);
	result = fread(out_data, 1, file_size, fp);

	dlog_print(DLOG_INFO, TAG, "read result:%d", result);

	fclose(fp);

	*size = file_size;

	return out_data;
}

void handle_gallery_img_req(int img_id)
{
	int img_len = g_list_length(images);

	GVariant *v_img;

	char *img_name, *img_path, *img_src, *base64_img;
	int img_buf_size, width, height;
	size_t base64_img_len;

	char *main_json;

	if (img_len == 0) {
		dlog_print(DLOG_ERROR, TAG, " no imgs");
		return;
	}

	dlog_print(DLOG_DEBUG, TAG, "img id: %d", img_id);

	v_img = g_list_nth_data(images, img_id);

	if (v_img == NULL) {
		dlog_print(DLOG_DEBUG, TAG, "img data does not exist");
		return;
	}

	g_variant_get(v_img, "(ssuuuu)", &img_name, &img_path, &img_src, &img_buf_size, &width, &height);

	dlog_print(DLOG_DEBUG, TAG, "img_name: %s", img_name);

	base64_img = base64_encode((unsigned char *)img_src, img_buf_size, &base64_img_len);

	main_json = allocate_json(2048 + base64_img_len); // enough memory to hold all thumbnails

	main_json = compose_image_header(main_json, "gallery-image-rsp", 1);

	dlog_print(DLOG_DEBUG, TAG, "json:%s", main_json);

	main_json = compose_fullscale_image(main_json, base64_img, base64_img_len, img_id, img_name, width, height);

	dlog_print(DLOG_DEBUG, TAG, "1");

	free(base64_img);

	dlog_print(DLOG_DEBUG, TAG, "sending image");

	sap_gallery_send(main_json);

	dlog_print(DLOG_DEBUG, TAG, "image sent");
}

void handle_gallery_thumbnail_req()
{
	int img_len = g_list_length(images);
	int prev_counter;
	char *main_json = NULL;

	long total_buf_size = 0;
	int counter1;

	if (img_len == 0) {
		dlog_print(DLOG_ERROR, TAG, " no imgs");
		return;
	}

	if (counter >= img_len) {
		counter = 0;
	}

	prev_counter = counter;

	for (counter1 = counter; counter1 < (counter + 3); counter1++) {

		char *t_name, *t_path, t_width, t_height;
		long img_size = 0;
		GVariant *v_tb_img = g_list_nth_data(tb_images, counter1);

		if (v_tb_img)
			g_variant_get(v_tb_img, "(ssuuu)", &t_name, &t_path, &t_width, &t_height, &img_size);

		dlog_print(DLOG_DEBUG, TAG, "total img_size: %d", img_size);

		total_buf_size += img_size;
	}

	total_buf_size = 4 * ((total_buf_size + 2) / 3) + 2;  //base64

	dlog_print(DLOG_DEBUG, TAG, "total thumbnail size: %d", total_buf_size);

	dlog_print(DLOG_DEBUG, TAG, "img_len: %d, counter:%d", img_len, counter);

	for (; (counter < (prev_counter + 3)) && (counter < img_len); counter++) {

		GVariant *v_tb_img = g_list_nth_data(tb_images, counter);

		int width, height;
		char *base64img, *img_src, *img_name, *img_path;
		size_t img_buf_size, base64_len;
		int img_size;

		dlog_print(DLOG_DEBUG, TAG, "counter: %d", counter);

		if (v_tb_img == NULL) {
			if (prev_counter == counter)
				prev_counter++;
			continue;
		}

		g_variant_get(v_tb_img, "(ssuuu)", &img_name, &img_path, &width, &height, &img_size);

		dlog_print(DLOG_DEBUG, TAG, "img_name: %s, img_path: %s, width:%d, height:%d", img_name, img_path, width, height);

		img_src = read_image(img_path, &img_buf_size);

		if (img_buf_size == 0) {
			if (prev_counter == counter)
				prev_counter++;
			continue;
		}

		dlog_print(DLOG_DEBUG, TAG, "img_siz:%d", img_buf_size);

		base64img = base64_encode((unsigned char *)img_src, img_buf_size, &base64_len);
		free(img_src);

		dlog_print(DLOG_DEBUG, TAG, "base64_len:%d", base64_len);

		if (counter == prev_counter) {
			//allocate json string
			main_json = allocate_json(2048 + total_buf_size); // enough memory to hold all thumbnails

			if (main_json == NULL) {
				dlog_print(DLOG_DEBUG, TAG, "json is NULL);");
				break;
			}

			main_json = compose_image_header(main_json, "gallery-thumbnail-rsp", img_len);

			dlog_print(DLOG_DEBUG, TAG, "json; %s", main_json);
		}

		main_json = compose_downlscaled_image(main_json, base64img, base64_len, counter, img_name, width, height, prev_counter == counter);

		free(base64img);
	}

	if (main_json) {
		strcat(main_json, "]");
		strcat(main_json, "}");

		dlog_print(DLOG_DEBUG, TAG, "json; %s", main_json);

		sap_gallery_send(main_json);
	}
}

bool gallery_media_item_cb(media_info_h media, void *user_data)
{
	media_info_h new_media = NULL;
	char *media_id = NULL;
	char *media_name = NULL;
	char *media_path = NULL;
	media_content_type_e media_type;

	char *img_path, *thumbnail_path;
	char *img_src = NULL;
	size_t img_buf_size;

	media_info_clone(&new_media, media);
	media_info_get_file_path(media, &img_path);

	media_info_get_thumbnail_path(media, &thumbnail_path);

	//storing images.
	img_src = read_image(img_path, &img_buf_size);

	media_info_get_media_id(media, &media_id);
	media_info_get_media_type(media, &media_type);
	media_info_get_display_name(media, &media_name);
	media_info_get_file_path(media, &media_path);


	if (media_type == MEDIA_CONTENT_TYPE_IMAGE) {
		image_meta_h image_handle;
		int width = 0, height = 0;
		int ret;

		ret = media_info_get_image(media, &image_handle);
		if (ret != MEDIA_CONTENT_ERROR_NONE) {
			// Error handling
		} else {
			GVariant *v_tb_img;
			GVariant *v_img_data;

			image_meta_get_width(image_handle, &width);
			image_meta_get_height(image_handle, &height);

			if (img_src) {
				long thumbnail_size;

				dlog_print(DLOG_INFO, TAG, "media_cb: file_path: %s", img_path);
				v_img_data = g_variant_new("(ssuuuu)", media_name, img_path, img_src, img_buf_size, width, height);
				images = g_list_append(images, v_img_data);

				thumbnail_size = read_img_size(thumbnail_path);
				if (thumbnail_size == 0) {
					g_free(thumbnail_path);
					thumbnail_path = img_path;
					thumbnail_size = read_img_size(img_path);
				}

				v_tb_img = g_variant_new("(ssuuu)", media_name, thumbnail_path, width, height, thumbnail_size);
				//add thumbnail img info.
				tb_images = g_list_append(tb_images, v_tb_img);
			}

		}
	}

	media_info_destroy(new_media);

	return true;
}

int get_files_list()
{
#define BUFLEN 200

	char buf[BUFLEN] = { '\0' };
	int ret = MEDIA_CONTENT_ERROR_NONE;
	filter_h filter = NULL;
	media_content_collation_e collate_type = MEDIA_CONTENT_COLLATE_NOCASE;
	media_content_order_e order_type = MEDIA_CONTENT_ORDER_DESC;

	media_content_connect();

	media_filter_create(&filter);

	snprintf(buf, BUFLEN, "%s=%d", MEDIA_TYPE, MEDIA_CONTENT_TYPE_IMAGE);

	ret = media_filter_set_condition(filter, buf, collate_type);
	if (ret != MEDIA_CONTENT_ERROR_NONE) {
		media_filter_destroy(filter);
		dlog_print(DLOG_ERROR, TAG, "Failed to set condition");

		return ret;
	}

	ret = media_filter_set_order(filter, order_type, MEDIA_DISPLAY_NAME, collate_type);
	if (ret != MEDIA_CONTENT_ERROR_NONE) {
		media_filter_destroy(filter);
		dlog_print(DLOG_ERROR, TAG, "Failed to set order");

		return ret;
	}

	ret = media_info_foreach_media_from_db(filter, gallery_media_item_cb, NULL);
	if (ret != MEDIA_CONTENT_ERROR_NONE) {
		dlog_print(DLOG_ERROR, TAG, "media_info_foreach_media_from_db failed: %d", ret);
		media_filter_destroy(filter);
		return ret;
	}

	media_filter_destroy(filter);

	media_content_disconnect();

	return 0;
}


typedef struct _item_data {
	int index;
	Elm_Object_Item *item;
} item_data;



static char *_gl_title_text_get(void *data, Evas_Object *obj, const char *part)
{
	char buf[1024];

	snprintf(buf, 1023, "%s", "Gallery");

	return strdup(buf);
}

static char *_gl_sub_title_text_get(void *data, Evas_Object *obj, const char *part)
{
	char buf[1024];

	snprintf(buf, 1023, "%s", "Provider");

	return strdup(buf);
}


static void _gl_del(void *data, Evas_Object *obj)
{
	// FIXME: Unrealized callback can be called after this.
	// Accessing Item_Data can be dangerous on unrealized callback.
	item_data *id = data;
	if (id) free(id);
}

static Eina_Bool _naviframe_pop_cb(void *data, Elm_Object_Item *it)
{
	ui_app_exit();
	return EINA_FALSE;
}

static void create_list_view(appdata_s *ad)
{
	Evas_Object *genlist = NULL;
	Evas_Object *naviframe = ad->naviframe;
	Elm_Object_Item *nf_it = NULL;
	item_data *id = NULL;
	int index = 0;

	Elm_Genlist_Item_Class *titc = elm_genlist_item_class_new();
	Elm_Genlist_Item_Class *pitc = elm_genlist_item_class_new();
	Elm_Genlist_Item_Class *gic = elm_genlist_item_class_new();

	/* Genlist Title Item Style */
	titc->item_style = "title";
	titc->func.text_get = _gl_title_text_get;
	titc->func.del = _gl_del;

	gic->item_style = "groupindex";
	gic->func.text_get = _gl_sub_title_text_get;
	gic->func.del = _gl_del;

	pitc->item_style = "padding";

	/* Create Genlist */
	genlist = elm_genlist_add(naviframe);
	elm_genlist_mode_set(genlist, ELM_LIST_COMPRESS);

	/* Create Circle Genlist */
	ad->circle_genlist = eext_circle_object_genlist_add(genlist, ad->circle_surface);

	/* Set Scroller Policy */
	eext_circle_object_genlist_scroller_policy_set(ad->circle_genlist, ELM_SCROLLER_POLICY_OFF, ELM_SCROLLER_POLICY_AUTO);

	/* Activate Rotary Event */
	eext_rotary_object_event_activated_set(ad->circle_genlist, EINA_TRUE);


	/* Genlist Padding Item Style */


	/* Title Item Here */
	id = calloc(sizeof(item_data), 1);
	elm_genlist_item_append(genlist, titc, NULL, NULL, ELM_GENLIST_ITEM_GROUP, NULL, NULL);

	id = calloc(sizeof(item_data), 1);
	id->index = index++;
	id->item = elm_genlist_item_append(genlist, gic, id, NULL, ELM_GENLIST_ITEM_GROUP, NULL, NULL);

	/* Padding Item Here */
	elm_genlist_item_append(genlist, pitc, NULL, NULL, ELM_GENLIST_ITEM_NONE, NULL, ad);

	nf_it = elm_naviframe_item_push(naviframe, NULL, NULL, NULL, genlist, "empty");
	elm_naviframe_item_pop_cb_set(nf_it, _naviframe_pop_cb, ad->win);


}


static void create_base_gui(appdata_s *ad)
{

	Evas_Object *conform = NULL;

	/* Window */

	ad->win = elm_win_util_standard_add(PACKAGE, PACKAGE);
	elm_win_autodel_set(ad->win, EINA_TRUE);

	evas_object_smart_callback_add(ad->win, "delete,request", win_delete_request_cb, NULL);

	/* Conformant */
	conform = elm_conformant_add(ad->win);
	evas_object_size_hint_weight_set(conform, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_win_resize_object_add(ad->win, conform);
	evas_object_show(conform);

	/* Naviframe */
	ad->naviframe = elm_naviframe_add(conform);
	elm_object_content_set(conform, ad->naviframe);

	/* Eext Circle Surface*/
	ad->circle_surface = eext_circle_surface_naviframe_add(ad->naviframe);

	/* Main View */
	create_list_view(ad);

	eext_object_event_callback_add(ad->naviframe, EEXT_CALLBACK_BACK, eext_naviframe_back_cb, NULL);
	eext_object_event_callback_add(ad->naviframe, EEXT_CALLBACK_MORE, eext_naviframe_more_cb, NULL);

	/* Show window after base gui is set up */
	evas_object_show(ad->win);

	get_files_list();
	initialize_sap();
}

static bool app_create(void *data)
{
	/* Hook to take necessary actions before main event loop starts
	        Initialize UI resources and application's data
	        If this function returns true, the main loop of application starts
	        If this function returns false, the application is terminated */
	appdata_s *ad = data;

	create_base_gui(ad);

	return true;
}

static void app_control(app_control_h app_control, void *data)
{
	/* Handle the launch request. */
}

static void app_pause(void *data)
{
	/* Take necessary actions when application becomes invisible. */
}

static void app_resume(void *data)
{
	/* Take necessary actions when application becomes visible. */
}

static void app_terminate(void *data)
{
	/* Release all resources. */
}

static void ui_app_lang_changed(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_LANGUAGE_CHANGED*/
	char *locale = NULL;
	system_settings_get_value_string(SYSTEM_SETTINGS_KEY_LOCALE_LANGUAGE, &locale);
	elm_language_set(locale);
	free(locale);
	return;
}

static void ui_app_orient_changed(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_DEVICE_ORIENTATION_CHANGED*/
	return;
}

static void ui_app_region_changed(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_REGION_FORMAT_CHANGED*/
}

static void ui_app_low_battery(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_LOW_BATTERY*/
}

static void ui_app_low_memory(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_LOW_MEMORY*/
}

int main(int argc, char *argv[])
{
	appdata_s ad = { 0, };
	int ret = 0;

	ui_app_lifecycle_callback_s event_callback = { 0, };
	app_event_handler_h handlers[5] = { NULL, };

	event_callback.create = app_create;
	event_callback.terminate = app_terminate;
	event_callback.pause = app_pause;
	event_callback.resume = app_resume;
	event_callback.app_control = app_control;

	ui_app_add_event_handler(&handlers[APP_EVENT_LOW_BATTERY], APP_EVENT_LOW_BATTERY, ui_app_low_battery, &ad);
	ui_app_add_event_handler(&handlers[APP_EVENT_LOW_MEMORY], APP_EVENT_LOW_MEMORY, ui_app_low_memory, &ad);
	ui_app_add_event_handler(&handlers[APP_EVENT_DEVICE_ORIENTATION_CHANGED], APP_EVENT_DEVICE_ORIENTATION_CHANGED, ui_app_orient_changed, &ad);
	ui_app_add_event_handler(&handlers[APP_EVENT_LANGUAGE_CHANGED], APP_EVENT_LANGUAGE_CHANGED, ui_app_lang_changed, &ad);
	ui_app_add_event_handler(&handlers[APP_EVENT_REGION_FORMAT_CHANGED], APP_EVENT_REGION_FORMAT_CHANGED, ui_app_region_changed, &ad);
	ui_app_remove_event_handler(handlers[APP_EVENT_LOW_MEMORY]);

	ret = ui_app_main(argc, argv, &event_callback, &ad);
	if (ret != APP_ERROR_NONE) {
		dlog_print(DLOG_ERROR, TAG, "app_main() is failed. err = %d", ret);
	}

	return ret;
}
